# captcha server

This server is used to provide captchas for your website. The goal is not to use remote service, yet to provide an efficient way to limit bots.

## Warning

This service is far from perfect and is provided as-is. It's the same server used on optifur. Resources for captcha are not provided, you should use your own to limit the bot's range of action (if a bot learns how to answer optifur's captcha, it won't know how to do it on your site)

## How to use
The service loads the global configuration from the `CONFIG_PATH` environment variable. 

```json
{
    "address":{
        "address": "0.0.0.0",
        "port": "8080"
    },
	"captchatConfig":"/path/to/captcha/config",
	"captchatKey":"/optional/path/to/key/file",
	"captchatSharedKey":"/optional/path/to/shared/key/file"
}
```

`address` is the address on which the service will listen
`captchatConfig` is the path to the captcha config file (see next section)
`captchatKey` is an optional path to a key file. The file must contain a standard base 64 encoding of a 32 bits long key. If no key is provided, one will be generated. This is useful if you have several captcha services running as you want them to share the same key.
`captchatSharedKey` is an optional path to a shared key file. The file must contain a standard base 64 encoding of a key. If provided, the key will be used to sign a validation token that can then be provided to other services for validation.

The token is in the format `timestamp.signature` with `timestamp` the go timestamp in seconds of when the captcha got validated, and the signature is the base 64 encoding of the hmac-sha256 of the timestamp with the shared key.


## Captcha config file

Here is an example used for tests:

```json
{
    "assetFolder": "static/",
    "challenges": {
        "number_positive": {
            "kind": "visual",
            "lang": [
                "en",
                "fr"
            ],
            "metadata": {},
            "resourcePath": "visual/test",
            "goodResources": [
                {
                    "name": "(\\d+)_(?:.*)",
                    "constraintFunction": "$1 >= $vars.diceValue"
                }
            ],
            "badResources": [
                {
                    "name": "(\\d+)_(?:.*)",
                    "constraintFunction": "$1 < $vars.diceValue"
                }
            ],
            "vars": {
                "diceValue": {
                    "type": "int",
                    "min": 3,
                    "max": 18
                }
            }
        }
    }
}
```

- `assetFolder` is the path to the folder containing assets. Here they are stored in `static/`;
- `challenges` is a map of challenge names to challenge configuration. When a challenge is requested, it will be chosen in this list;
- `kind` is the kind of challenge requested. When designing this service I had `visual` and `audio` in mind, but you can do whatever you want;
- `lang` are language supported for this challenge. Image recognition will most likely be universal, but text will not. **Beware:** fallback language is not supported, in this example requesting a test with language `jp` will return an error;
- `metadata` is a map of string to string used to describe the challenge. Eg: `"want":"greater_or_equal"`;
- `resourcePath` is the path to resources you want to use in the `assetFolder`. Here resources will be fetched from `static/visual/test`;
- `vars` are variable definitions used in challenge generation. When a challenge is chosen, variables are generated and used to define good and bad answers;
-  `goodResources` and `badResources` are lists of objects defining how to recognize good or bad resources:
   - `name` is a regex, a resource must match the regex to be valid. Golang regex are used here. Note that you can use template syntax to insert vars: `{{.vars.myVariable}}`;
   - `constraintFunction` is an optional JS expression. If the resource matches the regex, this expression will be evaluated and should return a truthy value if the resource match or a falsy value in the other case. Here `$vars` contains variable values for this challenge, `$0` contains the whole regex match and `$1`, `$2`, ... contains capture group values;
   - `invert` is an optional boolean to invert the match so that here bad resource could be
        ```json
        {
        "name": "(\\d+)_(?:.*)",
        "constraintFunction": "$1 >= $vars.diceValue",
        "invert":true
        }
        ```

### Variables
Currently, variables can be of two sorts:

`int`: choose a random int between min and max both included

```json
"diceValue": {
    "type": "int",
    "min": 3,
    "max": 18
}
```

`enum`: choose a random value in provided choices

```json
"diceValue": {
    "type": "enum",
    "choices":["3","5","8","13","20"]
}

```