package main

import (
	"context"
	"errors"
	"fmt"
	"net/http"
	"os"
	"os/signal"
	"strconv"
	"syscall"

	"github.com/rs/zerolog"
	"github.com/rs/zerolog/log"
	"gitlab.bad-wolf.fr/optifur/captcha-server/config"
	"gitlab.bad-wolf.fr/optifur/captcha-server/router"
	"golang.org/x/sync/errgroup"
)

func main() {

	logHuman, _ := os.LookupEnv("LOG_HUMAN")
	if ok, err := strconv.ParseBool(logHuman); err == nil && ok {
		log.Logger = log.Output(zerolog.ConsoleWriter{Out: os.Stderr})
	}
	zerolog.DefaultContextLogger = &log.Logger

	ctx := context.Background()
	cfg := config.Load()

	g, ctx := errgroup.WithContext(ctx)

	router := router.InitRouter(ctx)

	g.Go(func() error {
		return router.Start(fmt.Sprintf("%s:%s", cfg.Address.Address, cfg.Address.Port))
	})

	go func() {
		c := make(chan os.Signal, 16)
		signal.Notify(c, os.Interrupt, syscall.SIGTERM)

		// Wait for a signal or a router to be done
		select {
		case <-c:
		case <-ctx.Done():
		}

		// Close remaining router
		if err := router.Shutdown(ctx); err != nil {
			log.Err(err).Msg("unable to shutdow router")
		}
	}()

	if err := g.Wait(); err != nil && errors.Is(err, http.ErrServerClosed) {
		log.Err(err).Msg("router stopped")
	}

}
