package router

import (
	"context"
	"net/http"

	"github.com/labstack/echo/v4"
	echoMiddleware "github.com/labstack/echo/v4/middleware"
	"gitlab.bad-wolf.fr/optifur/captcha-server/handler"
	"gitlab.bad-wolf.fr/optifur/echo-system/wrapper"
)

// InitRouter initialize the router
func InitRouter(ctx context.Context) *echo.Echo {

	captchaHandler := handler.NewCaptchaHandler()

	e := echo.New()
	e.Use(echoMiddleware.Logger())

	e.GET("/ping", func(c echo.Context) error {
		return c.String(http.StatusOK, "OK")
	})

	// Return a captcha to be solved
	e.GET("/captcha", wrapper.WrapInOut(captchaHandler.GetCaptcha))
	// Access to captcha ressource
	e.GET("/captcha/resources/:id", wrapper.WrapInOnly(captchaHandler.GetCaptchaResource))
	// Validate a captcha
	e.POST("/captcha", wrapper.WrapInOut(captchaHandler.ValidateCaptcha))

	return e
}
