package challenge

import "testing"

func ptr[T any](in T) *T {
	return &in
}

func TestResourceMatcher_Match(t *testing.T) {
	type args struct {
		path string
		vars map[string]any
	}
	tests := []struct {
		name    string
		matcher ResourceMatcher
		args    args
		want    bool
		wantErr bool
	}{
		{
			"Test OK simple string match",
			ResourceMatcher{
				Name: "potato",
			},
			args{
				path: "potato",
				vars: nil,
			},
			true,
			false,
		},
		{
			"Test OK simple string match inverted",
			ResourceMatcher{
				Name:   "potato",
				Invert: true,
			},
			args{
				path: "potato",
				vars: nil,
			},
			false,
			false,
		},
		{
			"Test OK simple string no match",
			ResourceMatcher{
				Name: "potato",
			},
			args{
				path: "fries",
				vars: nil,
			},
			false,
			false,
		},
		{
			"Test OK simple string no match inverted",
			ResourceMatcher{
				Name:   "potato",
				Invert: true,
			},
			args{
				path: "fries",
				vars: nil,
			},
			true,
			false,
		},
		{
			"Test OK re match",
			ResourceMatcher{
				Name: "po.*to",
			},
			args{
				path: "potato",
				vars: nil,
			},
			true,
			false,
		},
		{
			"Test OK dummy truthy function",
			ResourceMatcher{
				Name:               "potato",
				ConstraintFunction: ptr(`true`),
			},
			args{
				path: "potato",
				vars: nil,
			},
			true,
			false,
		},
		{
			"Test OK dummy falsy function",
			ResourceMatcher{
				Name:               "potato",
				ConstraintFunction: ptr(`false`),
			},
			args{
				path: "fries",
				vars: nil,
			},
			false,
			false,
		},
		{
			"Test OK function whole match falsy",
			ResourceMatcher{
				Name:               "potato",
				ConstraintFunction: ptr(`$0 === "potato"`),
			},
			args{
				path: "fries",
				vars: nil,
			},
			false,
			false,
		},
		{
			"Test OK function whole match truthy",
			ResourceMatcher{
				Name:               "potato",
				ConstraintFunction: ptr(`$0 === "potato"`),
			},
			args{
				path: "potato",
				vars: nil,
			},
			true,
			false,
		},
		{
			"Test OK function sub-match",
			ResourceMatcher{
				Name:               "po(.*)to",
				ConstraintFunction: ptr(`$1 === "ta"`),
			},
			args{
				path: "potato",
				vars: nil,
			},
			true,
			false,
		},
		{
			"Test OK function sub-match",
			ResourceMatcher{
				Name:               "po(.*)to",
				ConstraintFunction: ptr(`$1 === "ta"`),
			},
			args{
				path: "pototo",
				vars: nil,
			},
			false,
			false,
		},
		{
			"Test OK function sub-match complex truthy",
			ResourceMatcher{
				Name:               "po(.*)to",
				ConstraintFunction: ptr(`$1 > 4`),
			},
			args{
				path: "po5to",
				vars: nil,
			},
			true,
			false,
		},
		{
			"Test OK function sub-match complex, falsy",
			ResourceMatcher{
				Name:               "po(.*)to",
				ConstraintFunction: ptr(`$1 > 4`),
			},
			args{
				path: "po3to",
				vars: nil,
			},
			false,
			false,
		},
	}
	for _, tt := range tests {
		t.Run(tt.name, func(t *testing.T) {
			got, err := tt.matcher.Match(tt.args.path, tt.args.vars)
			if (err != nil) != tt.wantErr {
				t.Errorf("ResourceMatcher.Match() error = %v, wantErr %v", err, tt.wantErr)
				return
			}
			if got != tt.want {
				t.Errorf("ResourceMatcher.Match() = %v, want %v", got, tt.want)
			}
		})
	}
}
